<?php

namespace Drupal\wn_social_share\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a block to share content to third party social platform.
 * 提供一个可以将内容分享到第三方社交平台的区块。
 *
 * @Block(
 *   id = "wn_social_share_block",
 *   admin_label = @Translation("Will Nice Social Share Block"),
 *   category = @Translation("Will Nice")
 * )
 */
class WnSocialShareBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * Configuration object.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;

  /**
   * Constructs a new WnSocialShareBlock instance.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ConfigFactoryInterface $config_factory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->config = $config_factory->get('wn_social_share.config');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);
    $config = $this->getConfiguration();
    $form['url_type'] = [
      '#type' => 'radios',
      '#title' => $this->t('Sharing Content'),
      '#description' => $this->t('Do you want to share the page where the block is located or website?'),
      '#default_value' => $config['url_type'] != NULL ? $config['url_type'] : 'page',
      '#options' => [
        'page' => $this->t('Page'),
        'site' => $this->t('Site'),
      ],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->setConfigurationValue('class_name', $form_state->getValue('class_name'));
    $this->setConfigurationValue('url_type', $form_state->getValue('url_type'));
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $platforms = "";
    $selected_social_platforms = $this->config->get('social_platforms');
    if (!empty($selected_social_platforms)) {
      $tmp_platforms = [];
      foreach ($selected_social_platforms as $selected_social_platform) {
        // Get platform sequence value from config.
        // 从配置取出每个社交平台的顺序值。
        $tmp_index = $this->config->get("{$selected_social_platform}_index") != NULL ? $this->config->get("{$selected_social_platform}_index") : 1;
        $tmp_platforms[] = [
          'name' => $selected_social_platform,
          'index' => $tmp_index,
        ];
      }
      // Sort array according to index value.
      // 数组按字段index重新排序
      array_multisort(array_column($tmp_platforms, 'index'), SORT_ASC, $tmp_platforms);
      // Retrieve an array with a single column named 'name'.
      // 取出name单一列的数组。
      $arr_platforms = array_column($tmp_platforms, 'name');
      $platforms = join(',', $arr_platforms);
    }
    $block_config = $this->getConfiguration();
    $url_type = $block_config['url_type'] != NULL ? $block_config['url_type'] : 'page';
    $data = [
      'platforms' => $platforms,
      'url_type' => $url_type,
    ];
    $build = [
      '#theme' => 'wn_social_share',
      '#data' => $data,
    ];
    $build['#attached']['library'][] = 'wn_social_share/share';
    $build['#attached']['drupalSettings']['base_url'] = Url::fromRoute('<front>', [], ['absolute' => TRUE])
      ->toString();
    return $build;
  }

}
